﻿using Microsoft.EntityFrameworkCore;
using SushiBarContracts.BindingModels;
using SushiBarContracts.SearchModels;
using SushiBarContracts.StoragesContracts;
using SushiBarContracts.ViewModels;
using SushiBarDatabaseImplement.Models;
using System.Collections.Generic;

namespace SushiBarDatabaseImplement.Implements
{
    public class OrderStorage : IOrderStorage
    {
        public List<OrderViewModel> GetFullList()
        {
            using var context = new SushiBarDatabase();
            return context.Orders
                    .Include(o => o.Sushi)
                    .Select(o => o.GetViewModel)
                    .ToList();
        }
        public List<OrderViewModel> GetFilteredList(OrderSearchModel model)
        {
            using var context = new SushiBarDatabase();
            return context.Orders
                    .Include(o => o.Sushi)
                    .Where(o => 
                    (model.Id.HasValue && o.Id == model.Id) || 
                    (model.DateFrom.HasValue && model.DateTo.HasValue && model.DateFrom < o.DateCreate && o.DateCreate < model.DateTo))
                    .Select(o => o.GetViewModel)
                    .ToList();
        }
        public OrderViewModel? GetElement(OrderSearchModel model)
        {
            if (!model.Id.HasValue)
            {
                return null;
            }
            using var context = new SushiBarDatabase();
            return context.Orders
                .Include (o => o.Sushi)
                .FirstOrDefault(o => o.Id == model.Id)?.GetViewModel;
        }
        public OrderViewModel? Insert(OrderBindingModel model)
        {
            using var context = new SushiBarDatabase();
            var newOrder = Order.Create(context, model);
            if (newOrder == null)
            {
                return null;
            }
            context.Orders.Add(newOrder);
            context.SaveChanges();
            return newOrder.GetViewModel;
        }
        public OrderViewModel? Update(OrderBindingModel model)
        {
            using var context = new SushiBarDatabase();
            using var transaction = context.Database.BeginTransaction();
            try
            {
                var order = context.Orders.Include(o => o.Sushi).FirstOrDefault(rec =>
                rec.Id == model.Id);
                if (order == null)
                {
                    return null;
                }
                order.Update(model);
                context.SaveChanges();
                transaction.Commit();
                return order.GetViewModel;
            }
            catch
            {
                transaction.Rollback();
                throw;
            }
        }
        public OrderViewModel? Delete(OrderBindingModel model)
        {
            using var context = new SushiBarDatabase();
            var element = context.Orders
                .FirstOrDefault(o => o.Id == model.Id);
            if (element != null)
            {
                context.Orders.Remove(element);
                context.SaveChanges();
                return element.GetViewModel;
            }
            return null;
        }
    }
}
